@@ -1,12 +1,10 @@
-import os
-from operator import itemgetter
+from operator import itemgetter,mul
 import numpy as np
 from netcdf import netcdf_file
 import sys 
-from operator import mul
 from Scientific.IO import NetCDF
 
-from ncdfextract import nctypecode
+from ncdfextract import ncgettypecode
 from ncdfproc import nccopydimension,nccopyattrvar
 
 class SomeError(Exception):
@@ -53,12 +51,9 @@ def rwicecube(filestream,shp,refiter,dimiter,dimpos,refnoiter,dimnoiter,icecube,
     #      dimpos = (5,10,9)
     
     # extend so that structured arrays are read at once
-    
-
 
     lennoiter = long(1)
 
-
     for irefnoiter,erefnoiter in enumerate(refnoiter):
         lennoiter = lennoiter*len(dimnoiter[irefnoiter])
 
@@ -178,15 +173,44 @@ def readicecubeps(fstream,shp,refiter,dimiter,dimiterpos,refnoiter,dimnoiter,vty
 # we kunnen eens proberen om een variabele aan te maken met een vooraf gespecifieerde dimensie!
 
 def pcd(func,dnamsel,datin,datout,appenddim = False, maxmembytes = 10000000):
-    """ process binary data in order of specified dimensions
-      func: the function to be used
-    dnamsel: the dimensions on which the function needs to apply
-    datin: list of input [file,variable]-pairs
-    datin: list of output [file,variable]-pairs
-    
-      Warning! for now output variables will be re-opened in write mode!
+    """ purpose (see also README.md): process binary NetCDF data streams
+    func: the function/analysis to be applied on the data input streams
+          specified below. The arguments  of func correspond to the respective
+          data input streams. The output needs to be a numpy.array(), or a list
+          of numpy.array() of which each item corresponds to the data output
+          streams specified below.
+
+    dnamsel: the dimensions on which the function needs to be applied. The
+             function will be repeated along the other dimensions
+
+    datin: list of data input variables/streams e.g. 
+           [{'file':ncfile,'varname':'T',...},{'file':ncfile2,'varname':'P',...}]
+        possible dictionary keywords hereby:
+        - 'file': <pointer to Scientific.IO.NetCDF.NetCDFFile('filename','r')>, or a list of filenames that represent NetCDF files all with exactly the same (NetCDF) data structure
+        - 'varname': NetCDF variable
+        - 'daliases' (optional): aliases for dimensions to 'align' dimensions
+                                 for different data input variables
+        - 'predim' (optional): when 'file' is a filename list,. 'predim' will
+                               be the name of the dimension that represents the
+                               filelist. When not specified, the outer
+                               dimension of the netcdf files will be expanded
+                               only if it appears to have a length of 1. In all
+                               other cases, the name predim<0,1/2/3>... will be
+                               taken as extra dimensions.  
+        - 'dsel': select a subset of the data by specifying dimension indices, 
+                for example {'level' : range(0,5,1), 'lat' : range(50,60,1),
+                'lon' : range(70,80,1)} will select a subspace of the first 5
+                levels, and a inner horizontal domain.  
+
+    datout: list of data output data variables/streams, in a similar fashion
+            as datin
+
+    appenddim: append dnamsel with the inner dimensions of the data input
+               streams when possible. This will generally lead to an increased
+               performance. This option needs be supported by the func.
+
+    maxmembytes: the maximum amount of bytes that is allowed for the buffers to be read/written from/to the input/output data streams. 
     """
-    
     # obtain definitions of the variable stream input
     vsdin = [] # input variable stream definitions
     for idatin,edatin in enumerate(datin):
@@ -205,7 +229,7 @@ def pcd(func,dnamsel,datin,datout,appenddim = False, maxmembytes = 10000000):
 
                 vsdin[idatin]['dnams'].append(str(edim))
 
-	    print '1dnams', idatin,vsdin[idatin]['dnams']
+	    #print '1dnams', idatin,vsdin[idatin]['dnams']
             # this statement could be a problem when defining daliases for the file-list dimension?!!!
             predim = None
             if 'predim' in datin[idatin]:
@@ -224,7 +248,7 @@ def pcd(func,dnamsel,datin,datout,appenddim = False, maxmembytes = 10000000):
                         idimextra = idimextra + 1
                 vsdin[idatin]['dnams'].insert(0,predim)
                 vsdin[idatin]['dims'] = [len(datin[idatin]['file'])]+list(nctemp.variables[datin[idatin]['varname']].shape[:])
-	    print '2dnams', idatin,vsdin[idatin]['dnams']
+	    #print '2dnams', idatin,vsdin[idatin]['dnams']
 		
         else:
             # we assume a netcdf file
@@ -343,7 +367,7 @@ def pcd(func,dnamsel,datin,datout,appenddim = False, maxmembytes = 10000000):
         else:
             idnam = dnamsstd.index(ednam)+1
     
-    print ("dimsstd", dimsstd)
+    #print ("dimsstd", dimsstd)
     
     # dimsstd: list the specific output dimensions
     # if function dimension: data output dimension should be the same as the function output dimension, but this should be checked afterwards.
@@ -462,7 +486,7 @@ def pcd(func,dnamsel,datin,datout,appenddim = False, maxmembytes = 10000000):
         if vsdout[iddout]['dtype'] == None:
             # output netcdf variable does not exist... creating
             # why does this needs to be little endian????
-            vsdout[iddout]['dtype'] = '>'+nctypecode(ddout[iddout].dtype)
+            vsdout[iddout]['dtype'] = '>'+ncgettypecode(ddout[iddout].dtype)
     
             # try to copy dimension from data input
             for idnams,ednams in enumerate(vsdout[iddout]['dnams']):
@@ -589,7 +613,7 @@ def pcd(func,dnamsel,datin,datout,appenddim = False, maxmembytes = 10000000):
         for irefdstd,erefdstd in enumerate(vsdout[ivsdout]['refdstd']):
             arefsout[ivsdout][erefdstd] = irefdstd
     
-    print ("dnamsstd", dnamsstd)
+    #print ("dnamsstd", dnamsstd)
     if appenddim == True:
         membytes = 0
         # dsellen = len(dnamsel)
@@ -820,7 +844,7 @@ def pcd(func,dnamsel,datin,datout,appenddim = False, maxmembytes = 10000000):
             # The inner dimensions just have to be referenced so not in correct order.  We know that they will be read in the correct order in the end
             rwchunksizein[ivsdin] = rwchunksizein[ivsdin]*vsdin[ivsdin]['dims'][idim]
             idim = idim - 1
-    print ("rwchunksizeout", rwchunksizein)
+    #print ("rwchunksizein", rwchunksizein)
     
 
     rwchunksizeout = [1]*len(vsdout)
@@ -830,7 +854,7 @@ def pcd(func,dnamsel,datin,datout,appenddim = False, maxmembytes = 10000000):
             # The inner dimensions just have to be referenced so not in correct order.  We know that they will be read in the correct order in the end
             rwchunksizeout[ivsdout] = rwchunksizeout[ivsdout]*vsdout[ivsdout]['dims'][idim]
             idim = idim - 1
-    print ("rwchunksizein",rwchunksizeout)
+    #print ("rwchunksizein",rwchunksizeout)
     
     adimnoapplyout = []
     for ivsdout,evsdout in enumerate(vsdout):
